@@ -5,7 +5,7 @@
 #'
 #' @param formula a formula object describing the model to be estimated.
 #' @param data a \code{data.frame} or \code{matrix} holding a panel data set. If no \code{index} variables are provided, the panel must be balanced and ordered in the long format \eqn{\bold{Y}=(Y_1^\prime, \dots, Y_N^\prime)^\prime}, \eqn{Y_i = (Y_{i1}, \dots, Y_{iT})^\prime} with \eqn{Y_{it} = (y_{it}, x_{it}^\prime)^\prime}. Conversely, if \code{data} is not ordered or not balanced, \code{data} must include two index variables that declare the cross-sectional unit \eqn{i} and the time period \eqn{t} of each observation.
-#' @param index a character vector holding two strings. The first string denotes the name of the index variable identifying the cross-sectional unit \eqn{i}, and the second string represents the name of the variable declaring the time period \eqn{t}. In case of a balanced panel data set that is ordered in the long format, \code{index} can be left empty if the the number of time periods \code{n_periods} is supplied.
+#' @param index a character vector holding two strings. The first string denotes the name of the index variable identifying the cross-sectional unit \eqn{i} and the second string represents the name of the variable declaring the time period \eqn{t}. The data is automatically sorted according to the variables in \code{index}, which may produce errors when the time index is a character variable. In case of a balanced panel data set that is ordered in the long format, \code{index} can be left empty if the the number of time periods \code{n_periods} is supplied.
 #' @param n_periods the number of observed time periods \eqn{T}. If an \code{index} character vector is passed, this argument can be left empty. Default is \code{Null}.
 #' @param lambda the tuning parameter determining the strength of the penalty term. Either a single \eqn{\lambda} or a vector of candidate values can be passed. If a vector is supplied, a BIC-type IC automatically selects the best fitting \eqn{\lambda} value.
 #' @param d the polynomial degree of the B-splines. Default is 3.
@@ -98,6 +98,10 @@ tv_pagfl <- function(formula, data, index = NULL, n_periods = NULL, lambda, d =
   if (any(all.vars(formula[[3]]) == ".")) {
     if (!is.null(index)) {
       data_temp <- data[, !(colnames(data) %in% index)]
+      if (NCOL(data) == 3) {
+        data_temp <- as.data.frame(data_temp)
+        colnames(data_temp) <- colnames(data)[!(colnames(data) %in% index)]
+      }
     } else {
       data_temp <- data
     }
@@ -176,7 +180,7 @@ tv_pagfl <- function(formula, data, index = NULL, n_periods = NULL, lambda, d =
   lambdaList <- tv_pagfl_routine(
     y = y, X = X, X_const = X_const, d = d, M = M, i_index = i_index, t_index = t_index, N = N, p_const = p_const, lambda_vec = lambda, kappa = kappa,
     min_group_frac = min_group_frac, max_iter = max_iter, tol_convergence = tol_convergence, tol_group = tol_group,
-    varrho = varrho, rho = rho, parallel = parallel
+    varrho = varrho, rho = rho, parallel = parallel, verbose = verbose
   )
   # Estimate fixed effects
   fe_vec <- getFE(y = y, i_index = i_index, N = N, method = "PLS")
